@@ -1,3 +1,5 @@
+from compiler import ast
+
 def triple_quote(doc):
     return '"""%s"""' % doc.replace('"""', '\"\"\"')
 
@@ -34,8 +36,7 @@ class ASTVisitor(object):
     def visitStmt(self, node, stream):
         for node in node.nodes:
             self.visit(node, stream)            
-            if stream.clear == False:
-                stream.write('\n')
+            stream.write("")
             
     def visitIf(self, node, stream):
         for index, test in enumerate(node.tests):
@@ -76,10 +77,11 @@ class ASTVisitor(object):
         for index, ass in enumerate(tuple(node.nodes)):
             self.visit(ass, stream)
             if index < len(tuple(node.nodes)) - 1:
-                stream.out(", ")
+                stream.out(" = ")
         stream.out(" = ")
         self.visit(node.expr, stream)
-
+        stream.write("")
+        
     def visitAssName(self, node, stream):
         if node.flags == 'OP_DELETE':
             stream.out("del ")
@@ -102,7 +104,10 @@ class ASTVisitor(object):
                 stream.out(format_argnames(argnames[:-len(node.defaults)]))
                 for index, default in enumerate(node.defaults):
                     name = argnames[index-len(node.defaults)]
-                    stream.out(", %s=" % name)
+                    if len(argnames) > len(node.defaults) or index > 0:
+                        stream.out(", %s=" % name)
+                    else:
+                        stream.out("%s=" % name)
                     self.visit(default, stream)
             else:
                 stream.out(format_argnames(argnames))
@@ -120,7 +125,9 @@ class ASTVisitor(object):
         stream.write("):")
         stream.indentation += 1
         for statement in node.code:
-            self.visit(statement, stream)
+            if statement is not None:
+                self.visit(statement, stream)
+                stream.write("")
         stream.indentation -= 1
 
     def visitConst(self, node, stream):
@@ -138,7 +145,8 @@ class ASTVisitor(object):
         for arg in tuple(node.args)[:-1]:
             self.visit(arg, stream)
             stream.out(", ")
-        self.visit(tuple(node.args)[-1], stream)
+        if node.args:
+            self.visit(node.args[-1], stream)
         if node.star_args:
             if node.args:
                 stream.out(", *")
@@ -157,15 +165,22 @@ class ASTVisitor(object):
         for index, ass in enumerate(tuple(node)):
             if index == 0 and ass.flags == 'OP_DELETE':
                 stream.out("del ")
-            stream.out(ass.name)
+            if isinstance(ass, ast.AssName):
+                stream.out(ass.name)
+            else:
+                self.visit(ass, stream)
             if index < len(tuple(node)) - 1:
                 stream.out(", ")
 
     def visitTuple(self, node, stream):
+        stream.out("(")
         for index, item in enumerate(tuple(node)):
             self.visit(item, stream)
             if index < len(tuple(node)) - 1:
                 stream.out(", ")
+        if len(node.nodes) == 1:
+            stream.out(", ")
+        stream.out(")")
 
     def visitGenExpr(self, node, stream):
         stream.out("(")
@@ -438,6 +453,10 @@ class ASTVisitor(object):
             if index < len(node.items) - 1:
                 stream.out(", ")            
         stream.out('}')
+
+    def visitNot(self, node, stream):
+        stream.out("not ")
+        self.visit(node.expr, stream)
         
     visitAdd = binary('+')
     visitSub = binary('-')
